@@ -48,7 +48,6 @@ import static org.apache.fury.type.TypeUtils.isBoxed;
 import static org.apache.fury.type.TypeUtils.isPrimitive;
 import static org.apache.fury.util.Preconditions.checkArgument;
 
-import com.google.common.collect.ImmutableSet;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -74,7 +73,6 @@ import org.apache.fury.codegen.Expression.ListExpression;
 import org.apache.fury.codegen.Expression.Literal;
 import org.apache.fury.codegen.Expression.Reference;
 import org.apache.fury.codegen.Expression.Return;
-import org.apache.fury.codegen.Expression.StaticInvoke;
 import org.apache.fury.codegen.ExpressionUtils;
 import org.apache.fury.codegen.ExpressionVisitor.ExprHolder;
 import org.apache.fury.collection.Tuple2;
@@ -87,10 +85,10 @@ import org.apache.fury.resolver.ClassInfoHolder;
 import org.apache.fury.resolver.ClassResolver;
 import org.apache.fury.resolver.RefResolver;
 import org.apache.fury.serializer.CompatibleSerializer;
+import org.apache.fury.serializer.EnumSerializer;
 import org.apache.fury.serializer.ObjectSerializer;
 import org.apache.fury.serializer.PrimitiveSerializers.LongSerializer;
 import org.apache.fury.serializer.Serializer;
-import org.apache.fury.serializer.Serializers;
 import org.apache.fury.serializer.StringSerializer;
 import org.apache.fury.serializer.collection.AbstractCollectionSerializer;
 import org.apache.fury.serializer.collection.AbstractMapSerializer;
@@ -189,7 +187,7 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
 
   protected abstract String codecSuffix();
 
-  <T> T visitFury(Function<Fury, T> function) {
+  protected <T> T visitFury(Function<Fury, T> function) {
     return fury.getJITContext().asyncVisitFury(function);
   }
 
@@ -280,7 +278,7 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
         Fury.class, MemoryBuffer.class, fury.getRefResolver().getClass(), Platform.class);
     ctx.addImports(ClassInfo.class, ClassInfoHolder.class, ClassResolver.class);
     ctx.addImport(Generated.class);
-    ctx.addImports(LazyInitBeanSerializer.class, Serializers.EnumSerializer.class);
+    ctx.addImports(LazyInitBeanSerializer.class, EnumSerializer.class);
     ctx.addImports(Serializer.class, StringSerializer.class);
     ctx.addImports(ObjectSerializer.class, CompatibleSerializer.class);
     ctx.addImports(AbstractCollectionSerializer.class, AbstractMapSerializer.class);
@@ -441,8 +439,7 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
                   classInfo,
                   inlineInvoke(classResolverRef, "getClassInfo", classInfoTypeRef, clsExpr))));
     }
-    writeClassAndObject.add(
-        fury.getClassResolver().writeClassExpr(classResolverRef, buffer, classInfo));
+    writeClassAndObject.add(classResolver.writeClassExpr(classResolverRef, buffer, classInfo));
     writeClassAndObject.add(
         new Invoke(
             inlineInvoke(classInfo, "getSerializer", SERIALIZER_TYPE),
@@ -451,11 +448,7 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
             buffer,
             inputObject));
     return invokeGenerated(
-        ctx,
-        ImmutableSet.of(buffer, inputObject),
-        writeClassAndObject,
-        "writeClassAndObject",
-        false);
+        ctx, ofHashSet(buffer, inputObject), writeClassAndObject, "writeClassAndObject", false);
   }
 
   /**
@@ -527,12 +520,7 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
     if (sourcePublicAccessible(cls)) {
       return Literal.ofClass(cls);
     } else {
-      return new StaticInvoke(
-          ReflectionUtils.class,
-          "loadClass",
-          CLASS_TYPE,
-          beanClassExpr(),
-          Literal.ofString(cls.getName()));
+      return staticClassFieldExpr(cls, "__class__" + cls.getName().replace(".", "_"));
     }
   }
 
@@ -659,8 +647,7 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
                 new Assign(
                     classInfo,
                     inlineInvoke(classResolverRef, "getClassInfo", classInfoTypeRef, clsExpr))));
-        writeClassAction.add(
-            fury.getClassResolver().writeClassExpr(classResolverRef, buffer, classInfo));
+        writeClassAction.add(classResolver.writeClassExpr(classResolverRef, buffer, classInfo));
         serializer = new Invoke(classInfo, "getSerializer", "serializer", SERIALIZER_TYPE, false);
         serializer = new Cast(serializer, TypeRef.of(AbstractCollectionSerializer.class));
         writeClassAction.add(serializer, new Return(serializer));
@@ -668,7 +655,7 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
         serializer =
             invokeGenerated(
                 ctx,
-                ImmutableSet.of(buffer, collection),
+                ofHashSet(buffer, collection),
                 writeClassAction,
                 "writeCollectionClassInfo",
                 false);
@@ -687,7 +674,7 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
     actions.add(write);
     if (generateNewMethod) {
       return invokeGenerated(
-          ctx, ImmutableSet.of(buffer, collection, serializer), actions, "writeCollection", false);
+          ctx, ofHashSet(buffer, collection, serializer), actions, "writeCollection", false);
     }
     return actions;
   }
@@ -965,15 +952,14 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
                     classInfo,
                     inlineInvoke(classResolverRef, "getClassInfo", classInfoTypeRef, clsExpr))));
         // Note: writeClassExpr is thread safe.
-        writeClassAction.add(
-            fury.getClassResolver().writeClassExpr(classResolverRef, buffer, classInfo));
+        writeClassAction.add(classResolver.writeClassExpr(classResolverRef, buffer, classInfo));
         serializer = new Invoke(classInfo, "getSerializer", "serializer", SERIALIZER_TYPE, false);
         serializer = new Cast(serializer, TypeRef.of(AbstractMapSerializer.class));
         writeClassAction.add(serializer, new Return(serializer));
         // Spit this into a separate method to avoid method too big to inline.
         serializer =
             invokeGenerated(
-                ctx, ImmutableSet.of(buffer, map), writeClassAction, "writeMapClassInfo", false);
+                ctx, ofHashSet(buffer, map), writeClassAction, "writeMapClassInfo", false);
       }
     } else if (!AbstractMapSerializer.class.isAssignableFrom(serializer.type().getRawType())) {
       serializer = new Cast(serializer, TypeRef.of(AbstractMapSerializer.class), "mapSerializer");
@@ -984,7 +970,7 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
             jitWriteMap(buffer, map, serializer, typeRef),
             new Invoke(serializer, "write", buffer, map));
     if (generateNewMethod) {
-      return invokeGenerated(ctx, ImmutableSet.of(buffer, map), write, "writeMap", false);
+      return invokeGenerated(ctx, ofHashSet(buffer, map), write, "writeMap", false);
     }
     return write;
   }
