@@ -22,6 +22,7 @@ package org.apache.fury.builder;
 import static org.apache.fury.codegen.Code.LiteralValue.FalseLiteral;
 import static org.apache.fury.codegen.Expression.Invoke.inlineInvoke;
 import static org.apache.fury.codegen.ExpressionUtils.add;
+import static org.apache.fury.collection.Collections.ofHashSet;
 import static org.apache.fury.type.TypeUtils.OBJECT_ARRAY_TYPE;
 import static org.apache.fury.type.TypeUtils.OBJECT_TYPE;
 import static org.apache.fury.type.TypeUtils.PRIMITIVE_BYTE_ARRAY_TYPE;
@@ -32,8 +33,6 @@ import static org.apache.fury.type.TypeUtils.getRawType;
 import static org.apache.fury.type.TypeUtils.getSizeOfPrimitiveType;
 import static org.apache.fury.type.TypeUtils.isPrimitive;
 
-import com.google.common.collect.ImmutableSet;
-import com.google.common.collect.Iterables;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -87,13 +86,27 @@ public class ObjectCodecBuilder extends BaseObjectCodecBuilder {
 
   public ObjectCodecBuilder(Class<?> beanClass, Fury fury) {
     super(TypeRef.of(beanClass), fury, Generated.GeneratedObjectSerializer.class);
-    Collection<Descriptor> descriptors =
-        classResolver.getAllDescriptorsMap(beanClass, true).values();
+    Collection<Descriptor> descriptors;
+    boolean shareMeta = fury.getConfig().shareMetaContext();
+    if (shareMeta) {
+      descriptors =
+          visitFury(
+              f ->
+                  f.getClassResolver()
+                      .getClassDef(beanClass, true)
+                      .getDescriptors(classResolver, beanClass));
+    } else {
+      descriptors = fury.getClassResolver().getAllDescriptorsMap(beanClass, true).values();
+    }
     classVersionHash =
         new Literal(ObjectSerializer.computeVersionHash(descriptors), PRIMITIVE_INT_TYPE);
     DescriptorGrouper grouper =
         DescriptorGrouper.createDescriptorGrouper(
-            descriptors, false, fury.compressInt(), fury.compressLong());
+            fury.getClassResolver()::isMonomorphic,
+            descriptors,
+            false,
+            fury.compressInt(),
+            fury.compressLong());
     objectCodecOptimizer =
         new ObjectCodecOptimizer(beanClass, grouper, !fury.isBasicTypesRefIgnored(), ctx);
     if (isRecord) {
@@ -148,17 +161,12 @@ public class ObjectCodecBuilder extends BaseObjectCodecBuilder {
     }
     expressions.addAll(serializePrimitives(bean, buffer, objectCodecOptimizer.primitiveGroups));
     int numGroups = getNumGroups(objectCodecOptimizer);
-    for (List<Descriptor> group :
-        Iterables.concat(
-            objectCodecOptimizer.boxedWriteGroups,
-            objectCodecOptimizer.finalWriteGroups,
-            objectCodecOptimizer.otherWriteGroups)) {
-      if (group.isEmpty()) {
-        continue;
-      }
-      boolean inline = group.size() == 1 && numGroups < 10;
-      expressions.add(serializeGroup(group, bean, buffer, inline));
-    }
+    addGroupExpressions(
+        objectCodecOptimizer.boxedWriteGroups, numGroups, expressions, bean, buffer);
+    addGroupExpressions(
+        objectCodecOptimizer.finalWriteGroups, numGroups, expressions, bean, buffer);
+    addGroupExpressions(
+        objectCodecOptimizer.otherWriteGroups, numGroups, expressions, bean, buffer);
     for (Descriptor descriptor :
         objectCodecOptimizer.descriptorGrouper.getCollectionDescriptors()) {
       expressions.add(serializeGroup(Collections.singletonList(descriptor), bean, buffer, false));
@@ -169,6 +177,21 @@ public class ObjectCodecBuilder extends BaseObjectCodecBuilder {
     return expressions;
   }
 
+  private void addGroupExpressions(
+      List<List<Descriptor>> writeGroup,
+      int numGroups,
+      ListExpression expressions,
+      Expression bean,
+      Reference buffer) {
+    for (List<Descriptor> group : writeGroup) {
+      if (group.isEmpty()) {
+        continue;
+      }
+      boolean inline = group.size() == 1 && numGroups < 10;
+      expressions.add(serializeGroup(group, bean, buffer, inline));
+    }
+  }
+
   private int getNumGroups(ObjectCodecOptimizer objectCodecOptimizer) {
     return objectCodecOptimizer.boxedWriteGroups.size()
         + objectCodecOptimizer.finalWriteGroups.size()
@@ -277,7 +300,7 @@ public class ObjectCodecBuilder extends BaseObjectCodecBuilder {
       } else {
         expressions.add(
             objectCodecOptimizer.invokeGenerated(
-                ImmutableSet.of(bean, base, writerAddr), groupExpressions, "writeFields"));
+                ofHashSet(bean, base, writerAddr), groupExpressions, "writeFields"));
       }
     }
     Expression increaseWriterIndex =
@@ -383,7 +406,7 @@ public class ObjectCodecBuilder extends BaseObjectCodecBuilder {
       } else {
         expressions.add(
             objectCodecOptimizer.invokeGenerated(
-                ImmutableSet.of(bean, buffer, base), groupExpressions, "writeFields"));
+                ofHashSet(bean, buffer, base), groupExpressions, "writeFields"));
       }
     }
     return expressions;
@@ -431,17 +454,12 @@ public class ObjectCodecBuilder extends BaseObjectCodecBuilder {
     }
     expressions.addAll(deserializePrimitives(bean, buffer, objectCodecOptimizer.primitiveGroups));
     int numGroups = getNumGroups(objectCodecOptimizer);
-    for (List<Descriptor> group :
-        Iterables.concat(
-            objectCodecOptimizer.boxedReadGroups,
-            objectCodecOptimizer.finalReadGroups,
-            objectCodecOptimizer.otherReadGroups)) {
-      if (group.isEmpty()) {
-        continue;
-      }
-      boolean inline = group.size() == 1 && numGroups < 10;
-      expressions.add(deserializeGroup(group, bean, buffer, inline));
-    }
+    deserializeReadGroup(
+        objectCodecOptimizer.boxedReadGroups, numGroups, expressions, bean, buffer);
+    deserializeReadGroup(
+        objectCodecOptimizer.finalReadGroups, numGroups, expressions, bean, buffer);
+    deserializeReadGroup(
+        objectCodecOptimizer.otherReadGroups, numGroups, expressions, bean, buffer);
     for (Descriptor d : objectCodecOptimizer.descriptorGrouper.getCollectionDescriptors()) {
       expressions.add(deserializeGroup(Collections.singletonList(d), bean, buffer, false));
     }
@@ -467,6 +485,21 @@ public class ObjectCodecBuilder extends BaseObjectCodecBuilder {
     return expressions;
   }
 
+  private void deserializeReadGroup(
+      List<List<Descriptor>> readGroups,
+      int numGroups,
+      ListExpression expressions,
+      Expression bean,
+      Reference buffer) {
+    for (List<Descriptor> group : readGroups) {
+      if (group.isEmpty()) {
+        continue;
+      }
+      boolean inline = group.size() == 1 && numGroups < 10;
+      expressions.add(deserializeGroup(group, bean, buffer, inline));
+    }
+  }
+
   protected Expression buildComponentsArray() {
     return new StaticInvoke(
         Platform.class, "copyObjectArray", OBJECT_ARRAY_TYPE, recordComponentDefaultValues);
@@ -639,7 +672,7 @@ public class ObjectCodecBuilder extends BaseObjectCodecBuilder {
       } else {
         expressions.add(
             objectCodecOptimizer.invokeGenerated(
-                ImmutableSet.of(bean, heapBuffer, readerAddr), groupExpressions, "readFields"));
+                ofHashSet(bean, heapBuffer, readerAddr), groupExpressions, "readFields"));
       }
     }
     Expression increaseReaderIndex =
@@ -728,7 +761,7 @@ public class ObjectCodecBuilder extends BaseObjectCodecBuilder {
       } else {
         expressions.add(
             objectCodecOptimizer.invokeGenerated(
-                ImmutableSet.of(bean, buffer, heapBuffer), groupExpressions, "readFields"));
+                ofHashSet(bean, buffer, heapBuffer), groupExpressions, "readFields"));
       }
     }
     return expressions;
