@@ -54,6 +54,7 @@ public class Config implements Serializable {
   private final boolean deserializeUnexistedClass;
   private final boolean scalaOptimizationEnabled;
   private transient int configHash;
+  private final boolean deserializeNonexistentEnumValueAsNull;
 
   public Config(FuryBuilder builder) {
     language = builder.language;
@@ -82,6 +83,7 @@ public class Config implements Serializable {
     }
     asyncCompilationEnabled = builder.asyncCompilationEnabled;
     scalaOptimizationEnabled = builder.scalaOptimizationEnabled;
+    deserializeNonexistentEnumValueAsNull = builder.deserializeNonexistentEnumValueAsNull;
   }
 
   public Language getLanguage() {
@@ -100,6 +102,11 @@ public class Config implements Serializable {
     return stringRefIgnored;
   }
 
+  /** ignore Enum Deserialize array out of bounds return null. */
+  public boolean deserializeNonexistentEnumValueAsNull() {
+    return deserializeNonexistentEnumValueAsNull;
+  }
+
   /**
    * Whether ignore reference tracking of all time types registered in {@link TimeSerializers} and
    * subclasses of those types when ref tracking is enabled.
