@@ -19,16 +19,17 @@
 
 package org.apache.fury.resolver;
 
-import static org.apache.fury.meta.MetaStringEncoder.PACKAGE_DECODER;
-import static org.apache.fury.meta.MetaStringEncoder.PACKAGE_ENCODER;
-import static org.apache.fury.meta.MetaStringEncoder.TYPE_NAME_DECODER;
+import static org.apache.fury.collection.Collections.ofHashMap;
+import static org.apache.fury.meta.ClassDef.SIZE_TWO_BYTES_FLAG;
+import static org.apache.fury.meta.Encoders.PACKAGE_DECODER;
+import static org.apache.fury.meta.Encoders.PACKAGE_ENCODER;
+import static org.apache.fury.meta.Encoders.TYPE_NAME_DECODER;
 import static org.apache.fury.serializer.CodegenSerializer.loadCodegenSerializer;
 import static org.apache.fury.serializer.CodegenSerializer.loadCompatibleCodegenSerializer;
 import static org.apache.fury.serializer.CodegenSerializer.supportCodegenForJavaSerialization;
 import static org.apache.fury.type.TypeUtils.OBJECT_TYPE;
 import static org.apache.fury.type.TypeUtils.getRawType;
 
-import com.google.common.collect.ImmutableMap;
 import java.io.Externalizable;
 import java.io.IOException;
 import java.io.Serializable;
@@ -106,6 +107,7 @@ import org.apache.fury.serializer.ArraySerializers.UnexistedEnumArrayClassSerial
 import org.apache.fury.serializer.BufferSerializers;
 import org.apache.fury.serializer.CodegenSerializer.LazyInitBeanSerializer;
 import org.apache.fury.serializer.CompatibleSerializer;
+import org.apache.fury.serializer.EnumSerializer;
 import org.apache.fury.serializer.ExternalizableSerializer;
 import org.apache.fury.serializer.JavaSerializer;
 import org.apache.fury.serializer.JdkProxySerializer;
@@ -245,6 +247,7 @@ public class ClassResolver {
     private final Set<Class<?>> getClassCtx = new HashSet<>();
     private final Map<Class<?>, FieldResolver> fieldResolverMap = new HashMap<>();
     private final Map<Long, Tuple2<ClassDef, ClassInfo>> classIdToDef = new HashMap<>();
+    private final Map<Class<?>, ClassDef> currentLayerClassDef = new HashMap<>();
     // TODO(chaokunyang) Better to  use soft reference, see ObjectStreamClass.
     private final ConcurrentHashMap<Tuple2<Class<?>, Boolean>, SortedMap<Field, Descriptor>>
         descriptorsCache = new ConcurrentHashMap<>();
@@ -334,17 +337,19 @@ public class ClassResolver {
     if (fury.getConfig().registerGuavaTypes()) {
       GuavaCollectionSerializers.registerDefaultSerializers(fury);
     }
-    if (metaContextShareEnabled) {
-      addDefaultSerializer(
-          UnexistedMetaSharedClass.class, new UnexistedClassSerializer(fury, null));
-      // Those class id must be known in advance, here is two bytes, so
-      // `UnexistedClassSerializer.writeClassDef`
-      // can overwrite written classinfo and replace with real classinfo.
-      short classId =
-          Objects.requireNonNull(classInfoMap.get(UnexistedMetaSharedClass.class)).classId;
-      Preconditions.checkArgument(classId > 63 && classId < 8192, classId);
-    } else {
-      register(UnexistedSkipClass.class);
+    if (fury.getConfig().deserializeUnexistedClass()) {
+      if (metaContextShareEnabled) {
+        addDefaultSerializer(
+            UnexistedMetaSharedClass.class, new UnexistedClassSerializer(fury, null));
+        // Those class id must be known in advance, here is two bytes, so
+        // `UnexistedClassSerializer.writeClassDef`
+        // can overwrite written classinfo and replace with real classinfo.
+        short classId =
+            Objects.requireNonNull(classInfoMap.get(UnexistedMetaSharedClass.class)).classId;
+        Preconditions.checkArgument(classId > 63 && classId < 8192, classId);
+      } else {
+        register(UnexistedSkipClass.class);
+      }
     }
   }
 
@@ -480,6 +485,10 @@ public class ClassResolver {
     }
   }
 
+  public boolean isRegistered(Class<?> cls) {
+    return extRegistry.registeredClassIdMap.get(cls) != null;
+  }
+
   public Short getRegisteredClassId(Class<?> cls) {
     return extRegistry.registeredClassIdMap.get(cls);
   }
@@ -504,7 +513,7 @@ public class ClassResolver {
   /**
    * Mark non-inner registered final types as non-final to write class def for those types. Note if
    * a class is registered but not an inner class with inner serializer, it will still be taken as
-   * non-final to write class def, so that it can be deserialized by the peer still..
+   * non-final to write class def, so that it can be deserialized by the peer still.
    */
   public boolean isMonomorphic(Class<?> clz) {
     if (fury.getConfig().shareMetaContext()) {
@@ -799,10 +808,10 @@ public class ClassResolver {
       return classInfo.serializer.getClass();
     } else {
       if (cls.isEnum()) {
-        return Serializers.EnumSerializer.class;
+        return EnumSerializer.class;
       } else if (Enum.class.isAssignableFrom(cls) && cls != Enum.class) {
         // handles an enum value that is an inner class. Eg: enum A {b{}};
-        return Serializers.EnumSerializer.class;
+        return EnumSerializer.class;
       } else if (EnumSet.class.isAssignableFrom(cls)) {
         return CollectionSerializers.EnumSetSerializer.class;
       } else if (Charset.class.isAssignableFrom(cls)) {
@@ -1048,6 +1057,7 @@ public class ClassResolver {
 
   public ClassInfo getClassInfo(short classId) {
     ClassInfo classInfo = registeredId2ClassInfo[classId];
+    assert classInfo != null : classId;
     if (classInfo.serializer == null) {
       addSerializer(classInfo.cls, createSerializer(classInfo.cls));
       classInfo = classInfoMap.get(classInfo.cls);
@@ -1275,8 +1285,9 @@ public class ClassResolver {
               || serializer instanceof ObjectSerializer
               || serializer instanceof MetaSharedSerializer)) {
         classDef =
-            classDefMap.computeIfAbsent(classInfo.cls, cls -> ClassDef.buildClassDef(cls, fury));
+            classDefMap.computeIfAbsent(classInfo.cls, cls -> ClassDef.buildClassDef(fury, cls));
       } else {
+        // TODO(chaokunyang) support more types meta-share serialization
         classDef =
             classDefMap.computeIfAbsent(
                 classInfo.cls,
@@ -1285,7 +1296,7 @@ public class ClassResolver {
                         this,
                         cls,
                         new ArrayList<>(),
-                        ImmutableMap.of(META_SHARE_FIELDS_INFO_KEY, "false")));
+                        ofHashMap(META_SHARE_FIELDS_INFO_KEY, "false")));
       }
       metaContext.writingClassDefs.add(classDef);
     }
@@ -1339,7 +1350,7 @@ public class ClassResolver {
           classInfo = getMetaSharedClassInfo(classDef, cls);
           // Share serializer for same version class def to avoid too much different meta
           // context take up too much memory.
-          extRegistry.classIdToDef.put(classDef.getId(), Tuple2.of(classDef, classInfo));
+          putClassDef(classDef, classInfo);
         } else {
           classInfo = classDefTuple.f1;
         }
@@ -1366,6 +1377,9 @@ public class ClassResolver {
       Preconditions.checkNotNull(classId);
       return classInfo;
     }
+    if (clz.isArray() || cls.isEnum()) {
+      return getClassInfo(cls);
+    }
     Class<? extends Serializer> sc =
         fury.getJITContext()
             .registerSerializerJITCallback(
@@ -1406,13 +1420,19 @@ public class ClassResolver {
     buffer.readerIndex(classDefOffset);
     int numClassDefs = buffer.readVarUint32Small14();
     for (int i = 0; i < numClassDefs; i++) {
-      ClassDef readClassDef = ClassDef.readClassDef(buffer);
-      // Share same class def to reduce memory footprint, since there may be many meta context.
-      ClassDef classDef =
-          extRegistry.classIdToDef.computeIfAbsent(
-                  readClassDef.getId(), key -> Tuple2.of(readClassDef, null))
-              .f0;
-      metaContext.readClassDefs.add(classDef);
+      long id = buffer.readInt64();
+      long hash = id >>> 8;
+      Tuple2<ClassDef, ClassInfo> tuple2 = extRegistry.classIdToDef.get(hash);
+      if (tuple2 != null) {
+        int size =
+            (id & SIZE_TWO_BYTES_FLAG) == 0
+                ? buffer.readByte() & 0xff
+                : buffer.readInt16() & 0xffff;
+        buffer.increaseReaderIndex(size);
+      } else {
+        tuple2 = readClassDef(buffer, id);
+      }
+      metaContext.readClassDefs.add(tuple2.f0);
       // Will be set lazily, so even some classes doesn't exist, remaining classinfo
       // can be created still.
       metaContext.readClassInfos.add(null);
@@ -1420,6 +1440,33 @@ public class ClassResolver {
     buffer.readerIndex(readerIndex);
   }
 
+  private Tuple2<ClassDef, ClassInfo> readClassDef(MemoryBuffer buffer, long header) {
+    ClassDef readClassDef = ClassDef.readClassDef(this, buffer, header);
+    Tuple2<ClassDef, ClassInfo> tuple2 = extRegistry.classIdToDef.get(readClassDef.getId());
+    if (tuple2 == null) {
+      tuple2 = putClassDef(readClassDef, null);
+    }
+    return tuple2;
+  }
+
+  private Tuple2<ClassDef, ClassInfo> putClassDef(ClassDef classDef, ClassInfo classInfo) {
+    Tuple2<ClassDef, ClassInfo> tuple2 = Tuple2.of(classDef, classInfo);
+    extRegistry.classIdToDef.put(classDef.getId(), tuple2);
+    return tuple2;
+  }
+
+  public ClassDef getClassDef(Class<?> cls, boolean resolveParent) {
+    if (resolveParent) {
+      return classDefMap.computeIfAbsent(cls, k -> ClassDef.buildClassDef(fury, cls));
+    }
+    ClassDef classDef = extRegistry.currentLayerClassDef.get(cls);
+    if (classDef == null) {
+      classDef = ClassDef.buildClassDef(fury, cls, false);
+      extRegistry.currentLayerClassDef.put(cls, classDef);
+    }
+    return classDef;
+  }
+
   /**
    * Native code for ClassResolver.writeClass is too big to inline, so inline it manually.
    *
@@ -1487,7 +1534,6 @@ public class ClassResolver {
       MetaStringBytes packageBytes = metaStringResolver.readMetaStringBytesWithFlag(buffer, header);
       MetaStringBytes simpleClassNameBytes = metaStringResolver.readMetaStringBytes(buffer);
       classInfo = loadBytesToClassInfo(packageBytes, simpleClassNameBytes);
-
     } else {
       classInfo = registeredId2ClassInfo[(short) (header >> 1)];
     }
