@@ -70,8 +70,13 @@ public class Descriptor {
     descCache = CacheBuilder.newBuilder().weakKeys().softValues().concurrencyLevel(64).build();
   }
 
+  // All fields should not be mutable except as lazy load,
+  // because Descriptor is cached in `descCache`.
+  // And mutable fields may make some serializer read wrong field
+  // value such as `typeName`.
   private TypeRef<?> typeRef;
   private Class<?> type;
+  private final String typeName;
   private final String name;
   private final int modifier;
   private final String declaringClass;
@@ -81,6 +86,7 @@ public class Descriptor {
 
   public Descriptor(Field field, TypeRef<?> typeRef, Method readMethod, Method writeMethod) {
     this.field = field;
+    this.typeName = field.getType().getName();
     this.name = field.getName();
     this.modifier = field.getModifiers();
     this.declaringClass = field.getDeclaringClass().getName();
@@ -91,6 +97,7 @@ public class Descriptor {
 
   public Descriptor(TypeRef<?> typeRef, String name, int modifier, String declaringClass) {
     this.field = null;
+    this.typeName = typeRef.getRawType().getName();
     this.name = name;
     this.modifier = modifier;
     this.declaringClass = declaringClass;
@@ -101,6 +108,7 @@ public class Descriptor {
 
   private Descriptor(Field field, Method readMethod) {
     this.field = field;
+    this.typeName = field.getType().getName();
     this.name = field.getName();
     this.modifier = field.getModifiers();
     this.declaringClass = field.getDeclaringClass().getName();
@@ -111,6 +119,7 @@ public class Descriptor {
 
   private Descriptor(
       TypeRef<?> typeRef,
+      String typeName,
       String name,
       int modifier,
       String declaringClass,
@@ -118,6 +127,7 @@ public class Descriptor {
       Method readMethod,
       Method writeMethod) {
     this.typeRef = typeRef;
+    this.typeName = typeName;
     this.name = name;
     this.modifier = modifier;
     this.declaringClass = declaringClass;
@@ -126,8 +136,14 @@ public class Descriptor {
     this.writeMethod = writeMethod;
   }
 
-  public Descriptor copy(TypeRef<?> typeRef, Method readMethod, Method writeMethod) {
-    return new Descriptor(typeRef, name, modifier, declaringClass, field, readMethod, writeMethod);
+  public Descriptor copy(Method readMethod, Method writeMethod) {
+    return new Descriptor(
+        typeRef, typeName, name, modifier, declaringClass, field, readMethod, writeMethod);
+  }
+
+  public Descriptor copyWithTypeName(String typeName) {
+    return new Descriptor(
+        typeRef, typeName, name, modifier, declaringClass, field, readMethod, writeMethod);
   }
 
   public Field getField() {
@@ -158,6 +174,10 @@ public class Descriptor {
     return writeMethod;
   }
 
+  public String getTypeName() {
+    return typeName;
+  }
+
   /** Try not use {@link TypeRef#getRawType()} since it's expensive. */
   public Class<?> getRawType() {
     Class<?> type = this.type;
@@ -182,11 +202,18 @@ public class Descriptor {
   @Override
   public String toString() {
     final StringBuilder sb = new StringBuilder("Descriptor{");
+    sb.append("typeName=").append(typeName);
     sb.append("name=").append(name);
-    sb.append(", field=").append(field);
-    sb.append(", readMethod=").append(readMethod);
-    sb.append(", writeMethod=").append(writeMethod);
-    sb.append(", typeToken=").append(typeRef);
+    sb.append("modifier=").append(modifier);
+    if (field != null) {
+      sb.append(", field=").append(field.getDeclaringClass().getSimpleName()).append('.');
+    }
+    if (readMethod != null) {
+      sb.append(", readMethod=").append(readMethod);
+    }
+    if (writeMethod != null) {
+      sb.append(", writeMethod=").append(writeMethod);
+    }
     sb.append('}');
     return sb.toString();
   }
